import os
import sys

from utils_base import Console, File, Log

from utils_ai.core.ChatRole import ChatRole
from utils_ai.providers import ProviderFactory

log = Log("ChatApp")


class ChatApp:
    @staticmethod
    def set_profile(ai, profile_path: str):
        if os.path.exists(profile_path):
            profile_content = File(profile_path).read()
            ai.set_profile(profile_content)
            print(Console.note(f"{profile_path=}"))
        else:
            log.error(f"Profile not found: {profile_path}")
        print("")

    @staticmethod
    def run_iter(ai):
        message = input(">> ")
        print("")

        if len(message) == 0:
            return

        if message in ["quit", "exit", "q", "x", "exit()", "quit()"]:
            print(Console.note("Bye!"))
            print()
            sys.exit(0)

        if message.lower().startswith("draw:"):
            image_path = ai.draw(prompt=message[5:])
            if image_path:
                print(Console.note(image_path))
                print("")
                os.startfile(image_path)
            return

        reply = ai.chat(message)
        if reply:
            print(Console.note(reply))
            print("")

    @staticmethod
    def run(provider_name: str, profile_path: str):
        ai = ProviderFactory.from_name(provider_name)
        print("")
        print(Console.note(f"[{ai.NAME}/{ai.MODEL}]"))
        print("")

        ChatApp.set_profile(ai, profile_path)

        ai.append_message(ChatRole.system, "Begin with a greeting")
        reply = ai.get_chat_reply(ai.messages)
        print(Console.note(reply))
        print("")

        while True:
            ChatApp.run_iter(ai)



import os
import tempfile

from utils_base import Hash, Log
from utils_www import WWW

log = Log('GenericAIImage')


class GenericAIImage:
    def get_image_url(self, prompt: str) -> str:
        raise NotImplementedError

    def draw(self, prompt: str):
        dir_desktop = os.environ.get('DIR_DESKTOP') or tempfile.gettempdir()
        h = Hash.md5(prompt)[:8]
        image_path = os.path.join(dir_desktop, f'dalle-{h}.png')
        if os.path.exists(image_path):
            log.warn('Image already exists.')
            return image_path

        try:
            image_url = self.get_image_url(prompt)
            WWW.download_binary(image_url, image_path)
            return image_path
        except Exception as e:
            log.error(str(e))
            return None



import os

from openai import OpenAI

from utils_ai.generic_ai import GenericAI


class OpenAIProvider(GenericAI):
    NAME = "OpenAI"
    MODEL = "gpt-4o"
    IMAGE_MODEL = "dall-e-3"

    def __init__(self):
        super().__init__()
        self.client = OpenAI(api_key=os.getenv("OPENAI_API_KEY"))

    def get_chat_reply(self, messages: list) -> str:
        response = self.client.chat.completions.create(
            model=self.MODEL,
            messages=messages,
        )
        return response.choices[0].message.content

    def get_image_url(self, prompt: str) -> str:
        response = self.client.images.generate(
            model=self.IMAGE_MODEL,
            prompt=prompt,
            size="1024x1024",
            n=1,
        )
        return response.data[0].url
